;********************************************************************
;   Plug in Libraries for the by 12 architecture                    *
;   Written and Tested using MPASM V1.40                            *
;********************************************************************
;   Filename:		SWI2C12.ASM                                 *
;   Date:               05 November 1997                            *
;   File Version:       1.00                                        *
;********************************************************************
;   Functions:                                                      *
;                                                                   *
;			OpenSWI2C                                   *
;			getcSWI2C                                   *
;			putcSWI2C                                   *
;                       getnextcSWI2C                               *
;********************************************************************
;   Revision History:                                               *
;      V1.00 - Beta release of Peripheral Libraries                 *
;********************************************************************
;   Notes:                                                          *
; The remainder of the tris bits for the port must be specified.  This
; is required because the By12 architecture won't allow reading of  *
; the TRIS register for the port, nor setting individual bits in the *
; TRIS register.   The whole register gets set at once.             *
; *******************************************************************
; Change these definitions as needed and reassemble the library     *
;                                                                   *
        title "Software I2C functions for by12 architecture"
	list p=pic16c5x
#include <p16c5x.inc>

#define  I2CPORT    PORTA
#define  I2CSDAPIN  0
#define  I2CSCLPIN  1
#define  TRISMASK   0x04
; ******************************************************************
; Shouldn't need to change anything below here.
;
	if I2CPORT < PORTA || I2CPORT > 7  ; PORTC
		messg "Invalid port value"
	endif

	if (I2CSDAPIN > 7) || ((I2CPORT == PORTA) && (I2CSDAPIN > 4))
		messg "Invalid SDA pin value"
	endif

	if (I2CSCLPIN > 7) || ((I2CPORT == PORTA) && (I2CSCLPIN > 4))
		messg "Invalid SCL pin value"
	endif

sdamask = 0x01 << I2CSDAPIN
sclmask = 0x01 << I2CSCLPIN

_I2CMASKOUT equ TRISMASK & (~sdamask) & (~sclmask)
_I2CMASKIN  equ (TRISMASK | sdamask) & (~sclmask)  ; Sets data to input
_I2CMASKOFF equ TRISMASK | sdamask | sclmask  ; sets both lines to hi Z
;************************************************************
;           Register Definitions
;************************************************************
        udata
I2CSlave      res    1        ; slave address
I2CAddress    res    1        ; address counter
I2CTxbuf      res    1        ; data getting sent out
I2CRxbuf      res    1
;   exported register names
        global  I2CSlave
        global  I2CAddress
        global  I2CRxbuf

;   exported function names
        global  OpenSWI2C
        global  getcSWI2C
        global  putcSWI2C
        global  getnextcSWI2C

        udata_ovr             ; these are only used during calls to
                              ; these functions and therefore can
                              ; be overlaid with other temp vars.
_I2Ccount     res    1        ; bit counter
_I2Cdataout   res    1
_loops        res    1        ; delay loop counter
_loops2       res    1        ; delay loop counter
_bcount       res    1        ; overlaid with _loops2

        code
;
;*****************************************************************
;	Function Name:	OpenSWI2C                                *
;	Return Value:	none                                     *
;	Parameters:	None (parameters set at assembly time)   *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	5 words                                  *
;	Description:	Sets the TRIS register for the I2CPORT   *
;			such that the I2CSDAPIN and I2CSCLPINs   *
;                       are outputs, and the rest of the bits    *
;                       are as specified by TRISMASK.  Also sets *
;                       sets both lines high (idle condition).   *   
;*****************************************************************

OpenSWI2C
        movlw   _I2CMASKOUT
        tris    I2CPORT
        movlw   sdamask | sclmask
        movwf   I2CPORT
        retlw   0

;*************************************************************
;       BITOUT routine takes the bit of data in '_I2Cdataout' and
;       transmits it to the serial EE device
;*************************************************************
BITOUT  macro
        local   bitlow
        local   clkout

        btfss   _I2Cdataout,7           ; check for state of data bit to xmit
        goto    bitlow                  ; low? go set data line low 
        bsf     I2CPORT,I2CSDAPIN       ; high? set data line high 
        goto    clkout                  ; go toggle the clock

bitlow  bcf     I2CPORT,I2CSDAPIN       ; output a low bit     
        goto    clkout                  ; needed for delay
clkout  goto    $+1
        bsf     I2CPORT,I2CSCLPIN       ; set clock line high
        goto    $+1
        goto    $+1
        bcf     I2CPORT,I2CSCLPIN       ; return clock line low
    endm
;                                                              

;**************************************************************
;       BITIN routine reads one bit of data from the 
;       serial EE device and stores it in the bit 'di'
;**************************************************************
BITIN   macro
        bsf     I2CPORT,I2CSCLPIN       ; set clock line high
        bsf     I2CRxbuf,0              ; assume input bit is high
        btfss   I2CPORT,I2CSDAPIN       ; read the data bit
        bcf     I2CRxbuf,0              ; input bit was low, set 'di' accordingly
        bcf     I2CPORT,I2CSCLPIN       ; set clock line low
        endm;
;
;**************************************************************
;       Start Bit Subroutine
;       this routine generates a start bit
;       (Low going data line while clock is high)          
;**************************************************************
;
BSTART
;       movlw   _I2CMASKOUT
;       tris    I2CPORT         ; set data and clock lines for output
;       bsf     I2CPORT,I2CSDAPIN       ; make sure data is high
;       bsf     I2CPORT,I2CSCLPIN       ; set clock high
;       goto    $+1                     ; wait 4700nS (Tsu:sta)
;       goto    $+1
        bcf     I2CPORT,I2CSDAPIN       ; data line goes low during
                                        ; high clock for start bit
        goto    $+1                     ; wait 4000nS (Thd:sta)
        goto    $+1
        bcf     I2CPORT,I2CSCLPIN       ; start clock train
        retlw   0
         ;
         ;    End of Subroutine
;************************************************************
;       Stop Bit Subroutine
;       This routine generates a stop bit
;       (High going data line while clock is high)
;************************************************************
BSTOP 
        movlw   _I2CMASKOUT
        tris    I2CPORT         ; set data/clock lines as outputs
        bsf     I2CPORT,I2CSCLPIN    ; set clock high
        goto    $+1             ; wait 4000nS (Tsu:sto)
        nop
        bsf     I2CPORT,I2CSDAPIN   ; data goes high while clock high
        retlw  0
;
;           End of Subroutine
;****************************************************************
;       Transmit Data Subroutine
;       This routine takes the byte of data stored in the
;       'datao' register and transmits it to the serial EE device.
;       It will then send 1 more clock to the serial EE for the
;       acknowledge bit.  If the ack bit from the part was low
;       then the transmission was sucessful.  If it is high, then
;       the device did not send a proper ack bit and the ack
;       fail LED will be turned on.
;       State of the acknowledge bit is returned in I2CRxbuf,0
;****************************************************************
TX
        movlw   .8
        movwf   _I2Ccount               ; set the #bits to 8
        movlw   _I2CMASKOUT             ; set data,clock as outputs
        tris    I2CPORT
TXLP    BITOUT
        rlf     _I2Cdataout, F          ; rotate txbuf left
        decfsz  _I2Ccount, F            ; 8 bits done?
        goto    TXLP                    ; no - go again
        movlw   _I2CMASKIN              ; make sdata an input line
        tris    I2CPORT
        BITIN
        retlw   0
;                    
;****************************************************************
;       Receive data routine
;       This routine reads one byte of data from the part
;       into the 'I2CRxbuf' register.  It then sends a high 
;       ack bit to indicate that no more data is to be read
;****************************************************************
RX
        clrf    I2CRxbuf          ; clear input buffer
        movlw   .8             ; set # bits to 8
        movwf   _I2Ccount      
        bcf     STATUS,0       ; make sure carry bit is low
        movlw   _I2CMASKIN              ; make sdata an input line
        tris    I2CPORT
RXLP    rlf     I2CRxbuf, F       ; rotate datai 1 bit left
        BITIN          ; read a bit
        decfsz  _I2Ccount, F       ; 8 bits done?
        goto    RXLP           ; no, do another
        bsf     _I2Cdataout,7      ; set ack bit = 1
        movlw   _I2CMASKOUT             ; set data,clock as outputs
        tris    I2CPORT
        BITOUT         ; to finish transmission
        retlw   0
;
;
;*****************************************************************
;	Function Name:	getcSWI2C                                *
;	Return Value:	none                                     *
;	Parameters:	None (parameters set at assembly time)   *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	5 words                                  *
;	Description:	Sends the address to the I2C device and  *
;			reads back one byte.  Expects I2CSlave   *
;                       and I2CAddress to be set prior to making *
;                       the call to getcSWI2C.                   *
;*****************************************************************
getcSWI2C
;

        call    BSTART          ; generate start bit
                                ;
                                ; now send the write control byte and
                                ; address to set the pointer
                                ;
        movf   I2CSlave,w       ; get slave address (write mode)
        movwf   _I2Cdataout     ; into transmit buffer
        call    TX              ; and send it
        btfsc   I2CRxbuf,0      ; test for NAK
        goto    abort
        movf    I2CAddress,w    ; get word address
        movwf   _I2Cdataout     ; into transmit buffer
        call    TX              ; and send it
        btfsc   I2CRxbuf,0
        goto    abort
        call    BSTOP
                             ;
                             ; now read one byte from the part
                             ;
;
;*****************************************************************
;	Function Name:	getnextcSWI2C                            *
;	Return Value:	none                                     *
;	Parameters:	None (parameters set at assembly time)   *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	5 words                                  *
;	Description:	reads the next byte sequentially from the  *
;			I2C device.  Depends on the I2C device's   *
;                       internal address counter.  Address should  *
;                       be set initially with a call to getcSWI2C. *
;                       Sequential reads are faster using        *
;                       getnextcSWI2C because the address is not *
;                       resent.                                  *
;*****************************************************************
getnextcSWI2C
        call    BSTART      ; generate start bit
        movf    I2CSlave,w  ; get slave address and read mode
        iorlw   0x01        ; set read bit
        movwf   _I2Cdataout       ; into transmit buffer
        call    TX          ; and transmit it
        btfsc   I2CRxbuf,0
        goto    abort
        call    RX          ; read 1 byte from serial EE
        call    BSTOP       ; send stop bit to end transmission
        bcf     STATUS,C
        retlw   0
abort
        call    BSTOP       ; send stop bit to end transmission
        bsf     STATUS,C
        retlw   1
 
;
;*****************************************************************
;	Function Name:	putcSWI2C                                *
;	Return Value:	none                                     *
;	Parameters:	None (parameters set at assembly time)   *
;	RAM Usage:	0 bytes                                  *
;	ROM Usage:	5 words                                  *
;	Description:	Writes a byte to the I2C device.  First  *
;			sends the device wakeup command, then    *
;                       I2CAddress, and finally the byte to      *
;                       write.  Returns ack status in carry bit. *
;                       0-ACK, 1-NAK
;*****************************************************************;
putcSWI2C
        movwf   I2CTxbuf        ; save byte to write
        call    BSTART          ; generate start bit
        movf    I2CSlave,w      ; move slave address
        movwf   _I2Cdataout     ; into transmit buffer
        call    TX              ; and send it        
        btfsc   I2CRxbuf,0
        goto    abort
        movf    I2CAddress,w    ; move word address
        movwf   _I2Cdataout     ; into transmit buffer
        call    TX              ; and send it        
        btfsc   I2CRxbuf,0
        goto    abort
        movf    I2CTxbuf,w      ; move data byte
        movwf   _I2Cdataout     ; to tranmit buffer
        call    TX              ; and transmit it
        btfsc   I2CRxbuf,0
        goto    abort
        call    BSTOP           ; generate stop bit
        bcf     STATUS,C
        retlw   0
;
        end
